package com.subd.subd.Services;

import com.subd.subd.Exceptions.CarNotFoundException;
import com.subd.subd.Exceptions.ClientNotFoundException;
import com.subd.subd.Exceptions.OrderNotFoundException;
import com.subd.subd.Exceptions.PickUpPointNotFoundException;
import com.subd.subd.Models.Car;
import com.subd.subd.Models.Client;
import com.subd.subd.Models.Order;
import com.subd.subd.Models.PickUpPoint;
import com.subd.subd.Repositories.CarRepository;
import com.subd.subd.Repositories.ClientRepository;
import com.subd.subd.Repositories.OrderRepository;
import com.subd.subd.Repositories.PickUpPointRepository;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Optional;

@Service
public class OrderService {
    private final OrderRepository orderRepository;
    private final ClientRepository clientRepository;
    private final PickUpPointRepository pickUpPointRepository;
    private final CarRepository carRepository;
    public OrderService(OrderRepository orderRepository, ClientRepository clientRepository, PickUpPointRepository pickUpPointRepository, CarRepository carRepository) {
        this.orderRepository = orderRepository;
        this.clientRepository = clientRepository;
        this.pickUpPointRepository = pickUpPointRepository;
        this.carRepository = carRepository;
    }
    @Transactional
    public Order addOrder(Double value, Long clientId, Long sourcePickUpPointId, Long destPickUpPointId, Long carId) {
        final Client client = clientRepository.findById(clientId)
                .orElseThrow(() -> new ClientNotFoundException(clientId));
        final PickUpPoint sourcePickUpPoint = pickUpPointRepository.findById(sourcePickUpPointId)
                .orElseThrow(() -> new PickUpPointNotFoundException(sourcePickUpPointId));
        final PickUpPoint destPickUpPoint = pickUpPointRepository.findById(destPickUpPointId)
                .orElseThrow(() -> new PickUpPointNotFoundException(destPickUpPointId));
        final Car car = carRepository.findById(carId)
                .orElseThrow(() -> new CarNotFoundException(carId));
        final Order order = new Order(value, client, sourcePickUpPoint, destPickUpPoint, car);
        return orderRepository.save(order);
    }
    @Transactional(readOnly = true)
    public Order findOrder(Long id) {
        final Optional<Order> order = orderRepository.findById(id);
        return order.orElseThrow(() -> new OrderNotFoundException(id));
    }
    @Transactional(readOnly = true)
    public List<Order> findAllOrders() {
        return orderRepository.findAll();
    }
    @Transactional
    public Order updateOrder(Long id, Double value, Long clientId, Long sourcePickUpPointId, Long destPickUpPointId, Long carId) {
        final Order currentOrder = findOrder(id);
        if (value != null) {
            currentOrder.setValue(value);
        }
        if (clientId != null) {
            final Client client = clientRepository.findById(clientId)
                    .orElseThrow(() -> new ClientNotFoundException(clientId));
            currentOrder.setClient(client);
        }
        if (sourcePickUpPointId != null) {
            final PickUpPoint sourcePickUpPoint = pickUpPointRepository.findById(sourcePickUpPointId)
                    .orElseThrow(() -> new PickUpPointNotFoundException(sourcePickUpPointId));
            currentOrder.setSourcePickUpPoint(sourcePickUpPoint);
        }
        if (destPickUpPointId != null) {
            final PickUpPoint destPickUpPoint = pickUpPointRepository.findById(destPickUpPointId)
                    .orElseThrow(() -> new PickUpPointNotFoundException(destPickUpPointId));
            currentOrder.setDestPickUpPoint(destPickUpPoint);
        }
        if (carId != null) {
            final Car car = carRepository.findById(carId)
                    .orElseThrow(() -> new CarNotFoundException(carId));
            currentOrder.setCar(car);
        }
        return orderRepository.save(currentOrder);
    }
    @Transactional
    public Order deleteOrder(Long id) {
        final Order currentOrder = findOrder(id);
        orderRepository.delete(currentOrder);
        return currentOrder;
    }
    @Transactional
    public void deleteAllOrder() {
        orderRepository.deleteAll();
    }
}
